import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_write_mode_route.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

final List<String> _randomIndicative = [
  'aime',
  'aimes',
  'aime',
  'aimons',
  'aimez',
  'aiment',
];

final List<String> _randomPronouns = [
  'je',
  'tu',
  'il/elle',
  'nous',
  'vous',
  'ils/elles',
];

final Finder nextPersonKeyFinder = find.byKey(Key('nextPersonKey'));
final Finder expectedVerbFinder = find.byKey(Key('expectedVerb'));
final Finder actualVerbFinder = find.byKey(Key('actualVerb'));
final Finder statusIconFinder = find.byKey(Key('comparisonStatusIcon'));
final Finder iconFinder = find.byIcon(Icons.assignment_turned_in_outlined);

void main() {
  testWidgets('Test the state of the verb', (WidgetTester tester) async {
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(_randomIndicative, _randomPronouns, 'fr',
                'infinitive', 'translation');
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.writtenMode);
          })
        ],
        child: VerbIndicativeStudyWriteModeRoute(
          key: UniqueKey(),
        )));
    final Finder translationFinder = find.text("translation");
    expect(translationFinder, findsOneWidget);
    expect(expectedVerbFinder, findsOneWidget);
    Finder expectedVerbValueFinder = find.text('aime');
    expect(expectedVerbValueFinder, findsOneWidget);
    expect(actualVerbFinder, findsOneWidget);
    expect(statusIconFinder, findsOneWidget);
    expect(iconFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder, 's');
    await tester.pump();
    Finder wrongLetterFinder = find.text('s');
    expect(wrongLetterFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder, _randomIndicative[0]);
    await tester.pump();
    expect(nextPersonKeyFinder, findsOneWidget);
    Finder secondPersonExpectedFinder = find.text(_randomIndicative[1]);
    expect(secondPersonExpectedFinder, findsNothing);
    await tester.tap(nextPersonKeyFinder);
    await tester.pump();
    expect(secondPersonExpectedFinder, findsOneWidget);
    Finder toTestButton = find.byKey(Key('write_forward_button'));
    for (int i = 1; i < 5; i++) {
      await tester.enterText(actualVerbFinder, _randomIndicative[i]);
      await tester.pump();
      expect(nextPersonKeyFinder, findsOneWidget);
      await tester.tap(nextPersonKeyFinder);
      await tester.pump();
    }
    expect(toTestButton, findsOneWidget);
  });
}

void testIconColor(WidgetTester tester, Finder iconFinder, Color color) {
  expect((tester.firstWidget(iconFinder) as Icon).color, color);
}
